/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 *
 *                                                                                              *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              *
 *                  http://picosa.narod.ru                                                      *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:           osa_mikroc16.h                                                              *
 *                                                                                              *
 *  Compilers:      mikroC PRO 3.0 and higher                                                   *
 *                                                                                              *
 *  Programmer:     Timofeev Victor                                                             *
 *                  osa@pic24.ru, testerplus@mail.ru                                            *
 *                                                                                              *
 *  Description:    mikroC PRO specific definitions for PIC16                                   *
 *                                                                                              *
 *  History:        22.09.2009                                                                  *
 *                                                                                              *
 ************************************************************************************************
 */


#ifndef __OSAMIKROC16_H__
#define __OSAMIKROC16_H__



/************************************************************************************************
 *                                                                                              *
 *     Integer types definitions                                                                * 
 *                                                                                              *
 ************************************************************************************************/

typedef unsigned char   __osa_uint8;
typedef unsigned int    __osa_uint16;
typedef unsigned long   __osa_uint32;
typedef unsigned char   __osa_bool;

typedef __osa_uint8     _OST_INT_TYPE;
#define _OST_INT_MASK    7
#define _OST_INT_SHIFT   3
#define _OST_INT_SIZE    8



/************************************************************************************************
 *                                                                                              * 
 *     Location keywords                                                                        * 
 *                                                                                              * 
 ************************************************************************************************/

#define NEAR
#define bank0
#define bank1
#define bank2
#define bank3

#define __osa30_near

//#define _OS_CUR_FLAGS_IN_OS_state       // Work with tasks state througth OS_state variable
#define _OS_CUR_FLAGS_IN_FSR       // Work with tasks state througth OS_state variable



/************************************************************************************************
 *                                                                                              * 
 *     Processor specified definitions for system flags                                         * 
 *     (see OST_SYSTEM_FLAGS and OST_TCB definition in file "OSA.h")                            * 
 *                                                                                              * 
 ************************************************************************************************/

#define _OST_SYSTEM_FLAGS_PROC_SPEC                                                             \
                                                                                                \
        /*5*/   _OST_INT_TYPE   bTimeout    : 1; /* Event waiting timeout               */      \
        /*6*/   _OST_INT_TYPE   bGIE_CTemp  : 1; /* Saved GIE while in critical section */      \

#define _OST_TCB_PROC_SPEC      // Empty macro for this processor


/************************************************************************************************
 *                                                                                              *
 *     Registers definitions                                                                    *
 *                                                                                              *
 ************************************************************************************************/


extern volatile char    _fsr;
extern volatile char    _status;
extern volatile char    _indf;
extern volatile char    _pcl;
extern volatile char    _pclath;
extern volatile char    _intcon;

#define     _carry      CARRY_bit
#define     _zero       ZERO_bit
#define     _irp        IRP_bit
#define     _gie        GIE_bit


/************************************************************************************************
 *                                                                                              *
 *     Constants and types                                                                      *
 *                                                                                              *
 ************************************************************************************************/

#ifndef OS_SMSG_SIZE
#define OS_SMSG_SIZE    0
#endif

//------------------------------------------------------------------------------
#define _FSR_TYPE               __osa_uint8

//------------------------------------------------------------------------------

#define OS_CODE_POINTER_SIZE        2
#define OST_CODE_POINTER            __osa_uint16



/************************************************************************************************
 *                                                                                              *
 *     Platform specific macros                                                                 * 
 *                                                                                              *
 ************************************************************************************************/


#define     OS_CLRWDT()         asm {clrwdt }
#define     OS_SLEEP()          asm {sleep  }


//------------------------------------------------------------------------------
#ifdef _OS_CUR_FLAGS_IN_FSR
//------------------------------------------------------------------------------

    //------------------------------------------------------------------------------
    #define OS_CUR_FLAGS_DEFINED
    //------------------------------------------------------------------------------
    
    #define _OS_bTaskReady              ((NEAR OST_TASK_STATE*)&_indf)->bReady
    #define _OS_bTaskCanContinue        ((NEAR OST_TASK_STATE*)&_indf)->bCanContinue
    #define _OS_bTaskDelay              ((NEAR OST_TASK_STATE*)&_indf)->bDelay
    #define _OS_bTaskTimeout          (!((NEAR OST_TASK_STATE*)&_indf)->bDelay)
    #define _OS_bTaskEnable             ((NEAR OST_TASK_STATE*)&_indf)->bEnable
    #define _OS_cTaskPriority           ((NEAR OST_TASK_STATE*)&_indf)->cPriority
    #define _OS_bTaskPaused             ((NEAR OST_TASK_STATE*)&_indf)->bPaused

//------------------------------------------------------------------------------
#endif      // _OS_CUR_FLAGS_IN_FSR
//------------------------------------------------------------------------------




/************************************************************************************************
 *                                                                                              *
 *     Context switching macros                                                                 *
 *                                                                                              *
 ************************************************************************************************/


extern void __OS_JumpToTask (void);


#define _OS_JumpToTask()    __OS_JumpToTask()


#define _OS_SaveRetAddr()                           \
    asm {                                           \
        clrf    __status                    ;       \
    }                                               \

    

 
//------------------------------------------------------------------------------

extern void OS_SchedRetPoint (void);

    
//------------------------------------------------------------------------------
#define _OS_RETURN_SAVE()                               \
    {                                                   \
        asm{    movlw   $+7           };                \
        asm{    movwf   __fsr         };                \
        asm{    movlw   hi_addr($+0x5)};                \
        OS_SchedRetPoint();                             \
        asm{    return           };                     \
        asm{    nop              };                     \
        asm{    nop              };                     \
    }

//------------------------------------------------------------------------------
#define _OS_CLEAR_READY()                               \
    {                                                   \
        asm{    movlw   $+7                 };          \
        asm{    movwf   __fsr               };          \
        asm{    movlw   hi_addr($+0x8005)   };          \
        asm{    bcf     __pclath, 0         };          \
        OS_SchedRetPoint();                             \
        asm{    return           };                     \
        asm{    nop              };                     \
        asm{    nop              };                     \
    }
//------------------------------------------------------------------------------
#define _OS_CLEAR_READY_SET_CANCONTINUE()               \
    {                                                   \
        asm{    movlw   $+7                 };          \
        asm{    movwf   __fsr               };          \
        asm{    movlw   hi_addr($+0x8005)   };          \
        asm{    bsf     __pclath, 0         };          \
        OS_SchedRetPoint();                             \
        asm{    return           };                     \
        asm{    nop              };                     \
        asm{    nop              };                     \
    }
//------------------------------------------------------------------------------
#define _OS_RETURN_NO_SAVE()                            \
        asm{ return}                                    \

//------------------------------------------------------------------------------

void __OS_SET_FSR_CUR_TASK (void);

#define _OS_SET_FSR_CUR_TASK()      __OS_SET_FSR_CUR_TASK()
#define _OS_SET_FSR_CUR_TASK_W(w)   {__OS_SET_FSR_CUR_TASK();_fsr+=w;}

#define _OS_SET_IRP_CUR_TASK()                      \
         asm{    movlw    hi_addr(_OS_TaskVars)} ;  \
         asm{    iorlw    0x00                 } ;  \
         asm{    bcf      __status, irp        } ;  \
         asm{    btfss    __status, z          } ;  \
         asm{    bsf      __status, irp        } ;  \

#define _OS_CLR_IRP()   _irp = 0
 











//#define _OS_DTimersWork_DEFINED
//
//    #define __OS_DTimersWork()                                              \
//        {                                                                   \
//            _OS_DTimerWork = (OST_DTIMER*)&_OS_DTimers;                     \
//            while (_OS_DTimerWork->Flags.bNextEnable)                       \
//            {                                                               \
//              _OS_DTimerWork = (OST_DTIMER*)_OS_DTimerWork->Next;           \
//              _OS_DTimerWork->Timer++;                                      \
//              if (!(_OS_DTimerWork->Timer))                                 \
//                _OS_DTimerWork->Flags.bTimeout = 1;                         \
//            }                                                               \
//        }


/******************************************************************************************
 *
 *  WORK WITH DYNAMIC TIMERS in OS_Timer
 *
 ******************************************************************************************/

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_DTIMERS
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#endif  // OS_ENABLE_DTIMER
//------------------------------------------------------------------------------







 





/************************************************************************************************
 *                                                                                              * 
 *     Create, replace, reserve task specific services                                          * 
 *                                                                                              * 
 ************************************************************************************************/


/************************************************************************/
/*                                                                      */
/* Replace current task with new (current task will be deleted)         */
/*                                                                      */
/************************************************************************/

#define OS_Task_Replace(priority, TaskName)                                         \
    {                                                                               \
        _OS_SET_FSR_CUR_TASK();                                                     \
        _indf = priority | 0x48; /* bEnable  bReady */                             \
        _OS_CurTask->pTaskPointer = ((OST_CODE_POINTER)&(TaskName));                \
        _OS_Return();                                                               \
    }


/************************************************************************/
/*                                                                      */
/* Create task                                                          */
/*                                                                      */
/************************************************************************/

#define OS_Task_Create(priority, TaskName)                          \
    {                                                               \
        _OS_Task_Create(priority, (OST_CODE_POINTER)&(TaskName));   \
    }

/************************************************************************/
/*                                                                      */
/* Create task by indirrect address                                     */
/*                                                                      */
/************************************************************************/

// If task created indirrectly then it is nessesary to insert in
// the main():
// #pragma funcall main TaskName






/************************************************************************************************
 *                                                                                              *
 *                                     I N T E R R U P T S                                      * 
 *                                                                                              *
 ************************************************************************************************/


extern  char OS_DI (void);
#define OS_RI(cSavedInt)    {if (cSavedInt & 0x80) _gie = 1;}
#define OS_EI()             { _gie = 1;}



/************************************************************************************************
 *                                                                                              *
 *                                     C O N D I T I O N S                                      *
 *                                                                                              *
 ************************************************************************************************/

extern  _OST_INT_TYPE _OS_CheckEvent (_OST_INT_TYPE bEvent);

#define _OS_CHECK_EVENT(event) if (!_OS_CheckEvent(event)) _OS_Return()












#endif
