#ifdef __PCM__
#include <16F628a.H>
//#device CONST=READ_ONLY
//#device CONST=ROM
#endif

#ifdef __PCH__
#include <18F252.H>
//#device CONST=READ_ONLY
//#device CONST=ROM
#endif

#include <osa.h>

#ifdef __OSAPICC__
#include <pic.h>
#endif

#ifdef __OSAPICC18__
#include <pic18.h>
#endif

#ifdef __OSAMCC18__
#include <p18f242.h>
#endif


//******************************************************************************
//  Variables
//******************************************************************************

OST_QUEUE           m_queue;
OST_SMSG            m_smsg;
OST_MSG_CB          m_msg_cb;
OST_FLAG            m_flag;
OST_DTIMER          m_dt1, m_dt2;
OST_CSEM            m_csem;
OST_TASK_POINTER    m_task;


OST_MSG             msgbuf[3];


__osa_uint8         m_cCounterRec[10];
__osa_uint32        m_dwTime;

#ifdef __OSACCS__
const char DATA0[] = "aaa";
const char DATA1[] = "bbb";
#else
const char DATA[][4] = {"aaa", "bbb", "ccc", "ddd"};
#endif

const char DATA_TEXT[] = "Data";

//******************************************************************************
//  Prototypes
//******************************************************************************

#ifdef __OSAMIKROC__
#pragma funcall main Task_Send
#endif

void Task_Task0 (void);
void Task_Task1 (void);
void Task_Rec0 (void);
void Task_Rec1 (void);
void Task_Rec2 (void);
void Task_Rec_Csem (void);
void Task_Rec_Queue (void);

//******************************************************************************
//   
//   Interrupt services 
//  
//******************************************************************************




/********************************************************************************
 *
 *  TASK: Task_Send
 *
 *------------------------------------------------------------------------------
 *
 *  description:    Sending all events
 *
 ********************************************************************************/

void Task_Send (void)
{
    static char s_cCounter;

    s_cCounter = 0;

    for (;;)
    {
        s_cCounter ++;
        OS_Bsem_Set(0);
        OS_Bsem_Set(20);

        OS_Msg_Send(m_msg_cb, DATA_TEXT);
        OS_Smsg_Send(m_smsg, s_cCounter);

        #ifdef __OSACCS__
        OS_Queue_Send(m_queue, s_cCounter & 1? DATA0:DATA1);
        #else
        OS_Queue_Send(m_queue, DATA[s_cCounter & 3]);
        #endif

        OS_Csem_Signal(m_csem);

        OS_Yield();
    }
}


//******************************************************************************
//  
//  Timers and delays
// 
//******************************************************************************


//******************************************************************************
//  
//  Tasks
// 
//******************************************************************************

void Task_Rec0 (void)
{
    m_cCounterRec[0] = 0;
    for (;;)
    {
        OS_Bsem_Wait(0);
        m_cCounterRec[0]++;
    }    
}    

void Task_Rec1 (void)
{
    m_cCounterRec[1] = 0;
    for (;;)
    {
        OS_Bsem_Wait(20);
        m_cCounterRec[1]++;
    }    
}    

void Task_Rec2 (void)
{
    OST_SMSG smsg;
    m_cCounterRec[2] = 0;
    for (;;)
    {
        OS_Smsg_Wait(m_smsg, smsg);
        OS_Delay(smsg);
        m_cCounterRec[2]++;
    }    
}    

//******************************************************************************
void Task_Rec_Csem (void)
{
    for (;;)
    {
        OS_Csem_Wait(m_csem);
        OS_Delay(20);
        if (!m_task) OS_Task_Create(6, Task_Rec_Queue);
        m_cCounterRec[3]++;
    }
}


//******************************************************************************
void Task_Task1 (void)
{
    for (;;)
    {
        OS_Task_Replace(7, Task_Task0);
    }    
}

void Task_Task0 (void)
{
    OST_MSG msg;
    OST_CODE_POINTER cp;
    
    OS_Msg_Wait(m_msg_cb, msg);
    msg = m_msg_cb.msg;
    if (msg == DATA_TEXT)
    {
        cp = Task_Task1;
        OS_Task_Create_P(7, cp);
    }
    OS_Task_Stop();
}    

//******************************************************************************

void Task_Rec_Queue (void)
{
    OST_MSG msg;
    char p[4];
    char a;
    
    m_task = OS_GetCurTask();
    for (;;)
    {
        OS_Queue_Wait(m_queue, msg);
        p[0] = *msg;
        p[1] = *msg + 1;
        p[2] = *msg + 2;
        p[3] = *msg + 3;
    }    
}        


//******************************************************************************
//  
//  Waiting events
//  
//******************************************************************************




void main (void)
{
    OS_Init();

    OS_Task_Create(7, Task_Send);
    OS_Task_Create(0, Task_Rec0);
    OS_Task_Create(1, Task_Rec1);
    OS_Task_Create(2, Task_Rec2);
    OS_Task_Create(0, Task_Rec_Csem);
    OS_Task_Create(7, Task_Task0);
    OS_Task_Reserve(Task_Task1);
    OS_Task_Reserve(Task_Rec_Queue);
    

    OS_Queue_Create(m_queue, msgbuf, 3);
    OS_Csem_Create(m_csem);
    OS_Msg_Create(m_msg_cb);

    m_task = 0;

    OS_EI();

    for (;;)
    {
        OS_Sched();
        OS_Timer();
        m_dwTime++;
        OS_CLRWDT();
    }
        
}
